/*
*    ZeroTrace: Oblivious Memory Primitives from Intel SGX
*    Copyright (C) 2018  Sajin (sshsshy)
*
*    This program is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, version 3 of the License.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#ifndef __OASM_LIB__
	#define __OASM_LIB__

  #ifndef BEFTS_MODE
    #include "../CONFIG.h"
    #include "Enclave_globals.h"
  #endif

  // Oblivious Buffer move/swap functions:
	extern "C" void oswap_buffer_16x(unsigned char *dest, unsigned char *source, uint32_t buffersize, uint8_t flag);
	extern "C" void oswap_buffer_byte(unsigned char *dest, unsigned char *source, uint32_t buffersize, uint8_t flag);

	extern "C" void oswap_buffer_byte_16x(unsigned char *dest, unsigned char *source, uint32_t buffersize, uint8_t flag);
	extern "C" void oswap_buffer_byte_v2(unsigned char *dest, unsigned char *source, uint8_t flag);

	extern "C" void ogt_comp_swap(uint64_t *key1, uint64_t *key2, unsigned char *buff1, unsigned char *buff2, uint32_t buffersize);

  enum OSwap_Style { OSWAP_4, OSWAP_8, OSWAP_12, OSWAP_16X, OSWAP_8_16X };
  template<OSwap_Style oswap_style> inline void oswap_buffer(unsigned char *dest, unsigned char *source, uint32_t buffersize, uint8_t flag);

  template<OSwap_Style oswap_style> inline void omove_buffer(unsigned char *dest, unsigned char *source, uint32_t buffersize, uint8_t flag);

  inline uint8_t ogt_set_flag(uint64_t key1, uint64_t key2)
  {
    uint8_t flag;
    __asm__ (
          "# inline ogt_set_flag\n"
          "cmp %[key2], %[key1]\n"
          //"# FOAV ogt_set_flag key1 (%[key1]):\n"
          "seta %[flag]\n"
          : [flag] "=r" (flag)
          : [key1] "r" (key1), [key2] "r" (key2)
          : "cc"
      );
    return flag;
  }

  inline uint8_t oge_set_flag(uint64_t key1, uint64_t key2)
  {
    uint8_t flag;
    __asm__ (
          "# inline oge_set_flag\n"
          "cmp %[key2], %[key1]\n"
          //"# FOAV oge_set_flag key1 (%[key1]):\n"
          "setae %[flag]\n"
          : [flag] "=r" (flag)
          : [key1] "r" (key1), [key2] "r" (key2)
          : "cc"
      );
    return flag;
  }


	inline void oset_value(uint64_t *dest, uint64_t value, uint32_t flag)
  {
    __asm__ (
        "# inline oset_value\n"
        "mov (%[dest]), %%r10\n"
        "test %[flag], %[flag]\n"
        "cmovnz %[value], %%r10\n"
        "mov %%r10, (%[dest])\n"
        :
        : [dest] "r" (dest), [value] "r" (value), [flag] "r" (flag)
        : "cc", "memory", "r10" 
    ); 
  }

  #include "oasm_lib.tcc"

#endif
